--------------------------------------------------------------------
-- PLG Core
--------------------------------------------------------------------

---------------------------
-- notes
---------------------------
-- figure out how to use debugging to parse through data line by line to check for errors.

---------------------------
-- set, reset recipe
---------------------------
function PLG:setRecipe(id,name,color,icon,link)
	PLG:Debug("Function","setRecipe"..id..": "..tostring(link))
	color = color or PLG.colors.ORANGE -- default color
	if id ~= PLG.selectedRecipe then -- gray for alternate
		color = PLG.colors.GRAY
	end	
	if name then -- set name
		if id==1 then -- set recipe title
			PLG:setText("Recipe"..id.."_Title",PLG.labels.RECIPE)
		else -- set alternate title
			PLG:setText("Recipe"..id.."_Title",PLG.labels.ALTERNATE)
		end
		PLG:setText("Recipe"..id.."_Text",color..name)
	else
		PLG:setText("Recipe"..id.."_Title","")
		PLG:setText("Recipe"..id.."_Text","")
	end
	if icon then -- set icon
		PLG["Recipe"..id].texture:SetTexture(icon)		
		if id ~= PLG.selectedRecipe then -- shade when not selected
			PLG["Recipe"..id].texture:SetVertexColor(0.5, 0.5, 0.5)
		else
			PLG["Recipe"..id].texture:SetVertexColor(1, 1, 1)
		end
		if link then -- set link
			PLG["Recipe"..id]:SetScript("OnEnter",function() 
				GameTooltip:SetOwner(PLG["Recipe"..id], "ANCHOR_TOPLEFT");
				GameTooltip:SetHyperlink(link);				
				if id ~= PLG.selectedRecipe then -- cursor and click note on alternate
					SetCursor("CAST_CURSOR")
					GameTooltip:AddLine(PLG.labels.SELECT,1,1,1,0)
				end
				GameTooltip:Show(); 
			end)
			PLG["Recipe"..id]:SetScript("OnLeave",function() 
				SetCursor("POINT_CURSOR")
				GameTooltip:Hide() 
			end)			
		else -- no link
			PLG["Recipe"..id]:SetScript("OnEnter",function() end)
		end
		if (PLG["Recipe1"].texture:GetTexture() and PLG["Recipe2"].texture:GetTexture())then -- two recipes, add onclick
			PLG["Recipe1"]:SetScript("OnClick",function() 
				PLG.selectedRecipe = 1
				PLG:Update()
			end)
			PLG["Recipe2"]:SetScript("OnClick",function() 
				PLG.selectedRecipe = 2
				PLG:Update()
			end)
		end
	else -- no icon
		PLG["Recipe"..id].texture:SetTexture(nil)
		PLG["Recipe"..id]:SetScript("OnEnter",function() end)
	end
end

---------------------------
-- set, reset reagent
---------------------------
function PLG:setReagent(id,text,count,icon,link)
	PLG:Debug("Function","setReagent"..id..": "..tostring(link))
	if icon then -- set the icon
		_G["PLG_Reagent"..id.."_Button"]:SetNormalTexture(icon)
		if link then -- set the tooltip
			_G["PLG_Reagent"..id.."_Button"]:SetScript("OnEnter",function() 
				GameTooltip:SetOwner(_G["PLG_Reagent"..id.."_Button"], "ANCHOR_TOPLEFT");
				GameTooltip:SetHyperlink(link);
				GameTooltip:Show(); 
			end)
			_G["PLG_Reagent"..id.."_Button"]:SetScript("OnLeave",function() 
				GameTooltip:Hide() 
			end)
		else -- no tooltip, kill onenter
			_G["PLG_Reagent"..id.."_Button"]:SetScript("OnEnter",function() end)
		end
		PLG:setText("Reagent_Title",PLG.labels.REAGENTS)
	else -- no icon
		PLG:setText("Reagent_Title","")
		_G["PLG_Reagent"..id.."_Button"]:SetNormalTexture(nil)
		_G["PLG_Reagent"..id.."_Button"]:SetScript("OnEnter",function() end)
	end
	if text then -- set text
		if count then -- set count
			PLG:setText("Reagent"..id.."_Text",text.."\n("..count..")")
		else
			PLG:setText("Reagent"..id.."_Text",text)
		end
	else -- no text
		PLG:setText("Reagent"..id.."_Text","")
	end
end

---------------------------
-- reset the text and icons
---------------------------
function PLG:resetDisplay()
	PLG:setRecipe(1,nil,nil,nil,nil)
	PLG:setRecipe(2,nil,nil,nil,nil)
	PLG:resetTooltip()
	for j=1,8 do
		PLG:setReagent(j,nil,nil,nil)
	end
	PLG:setText("Trainer","")
end

---------------------------
-- update the data
---------------------------
function PLG:Update()  --224/225 225/300
	PLG:Debug("Function","Update")
	PLG:resetDisplay()
	local sID, keep, npcID, questID, sTotal, pEarned, pTotal = unpack(PLG:getData())
	if sID[1] and PLG.lastSID ~= sID[1] then
		PLG.lastSID = sID[1]
		PLG.selectedRecipe = 1
		PLG.newRecipe = true
	end
	local hasSkill = nil
	if pEarned == pTotal then -- need to train
		if pTotal == PLG.maxLevel then -- max out (should never hit this)
			PLG:Debug("Condition","earned=total=max")
			PLG:setText("Instructions_Text",PLG.msgs.ERR_MAX_LEVEL)
		else --max on this level
			PLG:Debug("Condition","earned=total")
			PLG:setText("Instructions_Text",PLG.msgs.ERR_MAX_SKILL)
			PLG:setText("Trainer",PLG.colors.ORANGE..PLG.msgs.TRAINER)
		end
	else
		if not sID[1] then PLG:TRADE_SKILL_CLOSE(); return; end
		if pEarned > pTotal-25 then -- able to train
			PLG:Debug("Condition","earned=total-25")
			PLG:setText("Trainer",PLG.colors.ORANGE..PLG.msgs.TRAINER)
		end		
		if questID then -- quest tooltip
			PLG:setTooltip(PLG.labels.QUEST,function() PLG:getQuest(questID) end, questID)
		elseif npcID then -- vendor tooltip
			PLG:setTooltip(PLG.labels.NPC,function() PLG:getNPC(npcID) end, npcID)
		end
		for list=1,2 do
			local id = sID[list]
			local sName, sLink, sIcon, minLevel = unpack(PLG:getRecipe(id))
			if sName then
				if (PLG.pLevel >= minLevel) or PLG.debug then -- high enough level for this skill? or bypass if debugging
					for i=1, GetNumTradeSkills() do
						local skillName, skillType, _, _, _, numSkillUps, _ = GetTradeSkillInfo(i)
						if sName == skillName then	
							PLG:Debug("Condition","data matches skill"..list)	
							if not numSkillUps or numSkillUps < 1 then numSkillUps = 1 end -- can be 0 and gives a NaN result on division
							local need = math.floor((sTotal-pEarned)/numSkillUps) -- needed skillups till next recipe
							if keep > 0 then
								PLG:setText("Instructions_Text",PLG.msgs.CREATE.." "..PLG.msgs.KEEP,need,sTotal)
							else
								PLG:setText("Instructions_Text",PLG.msgs.CREATE,need,sTotal)
							end
							PLG:setRecipe(list,sName,PLG.SkillType[skillType],sIcon,sLink)							
							if PLG.selectedRecipe == list then 
								hasSkill = true
								PLG:getReagents(i,need)
								if PLG.newRecipe == true then
									TradeSkillFrame_SetSelection(i)
									TradeSkillFrame_Update();
									PLG.newRecipe = false
								end
							end
							break
						end
					end
					if not hasSkill then -- has not learned the skill
						if PLG.debug == true then
							local need = math.floor(sTotal-pEarned) -- needed skillups till next recipe
							if keep > 0 then
								PLG:setText("Instructions_Text",PLG.msgs.CREATE.." "..PLG.msgs.KEEP,need,sTotal)
							else
								PLG:setText("Instructions_Text",PLG.msgs.CREATE,need,sTotal)
							end
							PLG:setRecipe(list,sName,nil,sIcon,sLink)							
							if PLG.selectedRecipe == list then 
								if PLG.newRecipe == true then
									TradeSkillFrame_Update();
									PLG.newRecipe = false
								end
							end
						else
							PLG:Debug("Condition","unknown recipe")
							PLG:setText("Instructions_Text",PLG.msgs.ERR_LEARN)
							PLG:setRecipe(list,sName,nil,sIcon,sLink)
						end
					end
				else -- not high enough level (should never hit this.. I think...)
					PLG:Debug("Condition","high-level recipe")
					PLG:setText("Instructions_Text",PLG.msgs.ERR_LOW_LEVEL, minLevel)
					PLG:setRecipe(list,sName,nil,sIcon,sLink)
				end
			else
				if list==1 then
					PLG:setText("Instructions_Text",PLG.labels.WAITING)
					PLG:resetTooltip()
					PLG.waitForServer = true
				end
			end
		end
	end
end

---------------------------
-- get leveling data
---------------------------
function PLG:getData()
	PLG:Debug("Function","getData")
	local pSL, pEarned, pTotal, racial = unpack(PLG:getProfession())
	local data = PLG.Data[PLG.ProfessionNames[pSL]] -- dataset
	local sID1, sID2, npcID, quest1, Keep = nil,nil,nil,nil,nil
	if pEarned < PLG.maxLevel and data then
		-- load stored data
		for i=#data, 1, -1 do
			-- minlevel, maxlevel, recipe id, optional id, vendor type, vendor id both, vid horde, vid alliance
			local keep, sMin, sMax, rID1, rID2, vType, fIDb, fIDh, fIDa = unpack(data[i])
			if (pEarned >= (sMin + racial)) and (pEarned < (sMax + racial)) then -- earned greater than min AND earned less than max
				PLG:Debug("Condition","recipe found")
				sID1 = rID1 -- recipe/skill id
				sID2 = rID2 -- recipe/skill id
				sTotal = (data[i][3] + racial)
				if vType == 1 then -- vendor
					npcID = fIDb
					if not npcID then npcID = PLG:getCondition(PLG.pFaction == "Horde", fIDh, fIDa) end
				elseif vType == 2 then -- quest
					quest1 = fIDb
				end
				Keep = keep
			end
		end
	end
	return { {sID1, sID2}, Keep, npcID, quest1, sTotal, pEarned, pTotal }
end

---------------------------
-- get recipe info
---------------------------
function PLG:getRecipe(sID)
	PLG:Debug("Function","getRecipe")
	local sName, sLink, sIcon, minLevel = nil, nil, nil, 1
	if sID then
		local lID = strfind(tostring(sID), "s") or 0
		if lID > 0 then -- data item is a spell
			local val,_ = string.gsub(tostring(sID), "s", "")
			sID = tonumber(val)
			sName, _, sIcon, _ = GetSpellInfo(sID)
			sLink, _ = GetSpellLink(sID)
		else
			sName, sLink, _, _, minLevel, _, _, _, _, sIcon, _ = GetItemInfo(sID)
		end
	end
	return { sName, sLink, sIcon, minLevel or 1 }
end

---------------------------
-- get reagent info
---------------------------
function PLG:getReagents(recipe,need)
	PLG:Debug("Function","getReagents")
	for j=1,GetTradeSkillNumReagents(recipe) do -- get reagents
		local rName, rIcon, rCount, pCount = GetTradeSkillReagentInfo(recipe, j);
		local rLink = GetTradeSkillReagentItemLink(recipe, j)
		if rName ~= nil then
			PLG:setReagent(j,rName,pCount.."/"..(rCount*need),rIcon,rLink)
		end
	end
end

---------------------------
-- get quest data
---------------------------
function PLG:getQuest(id)
	PLG:Debug("Function","getQuest")
	local quest1, NPC1, npc1h, npc1a, quest2, NPC2, npc2h, npc2a = unpack(PLG.Vendors[2][id]) -- get quest data
	if not NPC1 then NPC1 = PLG:getCondition(PLG.pFaction == "Horde", npc1h, npc1a) end -- get NPC1 ID
	if not NPC2 then NPC2 = PLG:getCondition(PLG.pFaction == "Horde", npc2h, npc2a) end -- get NPC2 ID
	PLG:Tooltip(PLG.labels.QUEST1TITLE)
	PLG:Tooltip()
	PLG:Tooltip(quest1,nil,"YEL")
	PLG:Tooltip(PLG.msgs.QUEST1,nil,"WHT",nil,1)
	if NPC1 then
		PLG:Tooltip()
		local name, faction, location, x, y = unpack(PLG.Vendors[1][NPC1]) -- get NPC data
		PLG:Tooltip(PLG.labels.NPC, name, "WHT","GRN")
		PLG:Tooltip(PLG.labels.LOCATION, GetMapNameByID(location)..PLG.colors.WHITE.." ("..x..","..y..")", "WHT", "LBL")
	end
	if quest2 then
		PLG:Tooltip()
		PLG:Tooltip(PLG.labels.QUEST2TITLE,nil,"GLD")
		PLG:Tooltip(quest2,nil,"YEL")
		PLG:Tooltip(PLG.msgs.QUEST2,nil,"WHT",nil,1)
	end
	if NPC2 then
		PLG:Tooltip()
		local name, faction, location, x, y = unpack(PLG.Vendors[1][NPC2]) -- get NPC data
		PLG:Tooltip(PLG.labels.NPC, name, "WHT","GRN")
		PLG:Tooltip(PLG.labels.LOCATION, GetMapNameByID(location)..PLG.colors.WHITE.." ("..x..","..y..")", "WHT", "LBL")
	end
end

---------------------------
-- get npc data
---------------------------
function PLG:getNPC(id)
	PLG:Debug("Function","getNPC")
	local name, faction, location, x, y = unpack(PLG.Vendors[1][id]) -- get NPC data
	PLG:Tooltip(PLG.labels.VENDOR)
	PLG:Tooltip(PLG.msgs.VENDOR1,nil,"WHT",nil,1)
	PLG:Tooltip()
	PLG:Tooltip(PLG.labels.NPC, name, "WHT","GRN")
	PLG:Tooltip(PLG.labels.LOCATION, GetMapNameByID(location)..PLG.colors.WHITE.." ("..x..","..y..")", "WHT", "LBL")
end

---------------------------
-- set quest/npc link/tooltip
---------------------------
function PLG:setTooltip(text,func, id)
	PLG:Debug("Function","setTooltip")
	PLG:setText("Vendor_Button","%s[%s]",PLG.colors.BLUE,text)
	_G["PLG_Vendor_Button"]:SetWidth(_G["PLG_Vendor_Button"]:GetTextWidth())
	_G["PLG_Vendor_Button"]:SetScript("OnEnter",function() 
		SetCursor("CAST_CURSOR")
		PLG.tooltip:ClearLines()
		PLG.tooltip:ClearAllPoints()
		PLG.tooltip:SetOwner(_G["PLG_Vendor_Button"],"ANCHOR_NONE")
		PLG.tooltip:SetPoint("BOTTOMLEFT", _G["PLG_Vendor_Button"], "TOPLEFT", 0, 0)	
		func()
		PLG.tooltip:Show()
	end)
	_G["PLG_Vendor_Button"]:SetScript("OnLeave",function() 
		SetCursor("POINT_CURSOR")
		PLG.tooltip:Hide()
	end)
	_G["PLG_Vendor_Button"]:SetScript("OnClick",function() 
		if text == PLG.labels.QUEST then		
			local quest1, NPC1, npc1h, npc1a, _ = unpack(PLG.Vendors[2][id]) -- get quest data
			if not NPC1 then NPC1 = PLG:getCondition(PLG.pFaction == "Horde", npc1h, npc1a) end -- get NPC1 ID
			id = NPC1
		end
		local name, faction, location, x, y = unpack(PLG.Vendors[1][id])	
		if PLG.tomtom then
			TomTom:AddMFWaypoint(location, nil, x/100, y/100, {["title"] = name,})
		elseif PLG.carbonite then
			Nx.slashCommand("goto "..GetMapNameByID(location).." "..x.." "..y.." "..name)
		end
	end)
end

---------------------------
-- turn off vendor button
---------------------------
function PLG:resetTooltip()
	PLG:Debug("Function","resetTooltip")
	PLG:setText("Vendor_Button"," ")
	_G["PLG_Vendor_Button"]:SetWidth(_G["PLG_Vendor_Button"]:GetTextWidth())
	_G["PLG_Vendor_Button"]:SetScript("OnEnter",function() end)
	_G["PLG_Vendor_Button"]:SetScript("OnLeave",function() end)
	_G["PLG_Vendor_Button"]:SetScript("OnClick",function() end)
end

---------------------------
-- get profession info
---------------------------
function PLG:getProfession()
	PLG:Debug("Function","getProfession")
	local Profession = TradeSkillFrameTitleText:GetText() -- frame title
	local pSL = nil
	if PLG.debug== true then Profession = PLG.debugProfession end
	local pEarned, pTotal, racial = 0,0,0
	-- find the profession
	for i=1, #PLG.Professions do		
		if PLG.Professions[i] ~= nil then
			local pName, _, earned, total, _, _, skillLine, _ = GetProfessionInfo(PLG.Professions[i])
			if pName == Profession then -- earned/total ranks for profession
				PLG:Debug("Condition","profession found")
				pEarned = earned
				pTotal = total
				pSL = skillLine
				--if pSL == 185 then pSL = 186 end  (fake cooking = smelting)
			end
		end
	end
	if PLG.debug== true then 
		pEarned = PLG.debugEarned; 
		pTotal = PLG.debugTotal; 
		for k,v in pairs(PLG.ProfessionNames) do 
			if v == Profession then
				pSL = k
				PLG:Debug("Condition","profession faked")
				break
			end
		end
	end
	-- racial buffs
	if pSL and PLG.Racial[pSL] ~= nil then
		if IsSpellKnown(PLG.Racial[pSL][1]) == true then
			PLG:Debug("Condition","racial trait found")
			--racial = PLG.Racial[pSL][2]
		end
	end
	return {pSL, pEarned, pTotal, racial}
end

---------------------------
-- register events
---------------------------
function PLG:RegisterEvents()
	PLG:Debug("Function","RegisterEvents")
	for i=1, #PLG.events do
		PLG.frame:RegisterEvent(PLG.events[i])
	end
end

---------------------------
-- on show event handler
---------------------------
function PLG:TRADE_SKILL_SHOW(force)	
	PLG:Debug("Function","TRADE_SKILL_SHOW")
	PLG.selectedRecipe = 1
	local PSL, pEarned, pTotal, racial = unpack(PLG:getProfession())
	-- not maxed, not runeforging, not mining, not herbalism, not fishing, not skinning, not archaeology
	if ((pEarned < PLG.maxLevel) and (pTotal > 1) and (PSL ~= 182 and PSL ~= 186 and PSL ~= 356 and PSL ~= 393 and PSL ~= nil)) or force then 
		PLG:Debug("Condition","still leveling")
		if PLG.UseFrame then 
			PLG.backframe:Show()
		else
			PLG:MakeDouble()
		end
		PLG.frame:Show()
		PLG.newRecipe = true
		PLG:Update()	
	end
end

---------------------------
-- on close event handler
---------------------------
function PLG:TRADE_SKILL_CLOSE()
	PLG:Debug("Function","TRADE_SKILL_CLOSE")	
	if PLG.UseFrame then 
		PLG.backframe:Hide()
	else
		PLG:MakeSingle()
	end
	PLG.frame:Hide()
end

---------------------------
-- localization
---------------------------
function PLG:Localize()
	local locale = GetLocale()
	if locale == "enGB" then locale = "enUS" end -- GB defaults to US
	if PLG[locale] then -- if localized for this region
		PLG.msgs = PLG[locale].msgs
		PLG.labels = PLG[locale].labels
	else -- default to enUS
		PLG.msgs = PLG["enUS"].msgs
		PLG.labels = PLG["enUS"].labels
	end
end

---------------------------
-- init
---------------------------
function PLG:Initialize()
	PLG:Localize()
	PLG:Debug("Function","Initialize")
	
	-- check for addons
	for i=1, GetNumAddOns() do
		if IsAddOnLoaded(i) then
			local name, title = GetAddOnInfo(i)
			if name == "TomTom" then PLG.tomtom = true end
			if name == "Carbonite" then PLG.carbonite = true end
			if name == "TradeskillHD" then PLG.TradeSkillHD = true end
			if name == "TradeSkillDW" then PLG.TradeSkillDW = true end
			if name == "DoubleWideTradeSkills" then PLG.DoubleWideTradeSkills = true end
		end
	end
	
	-- move PLG outside the TradeSkillFrame
	if PLG.TradeSkillDW or PLG.DoubleWideTradeSkills or PLG.TradeSkillHD or PLG.debug then
		PLG.UseFrame = true
	end
	
	-- initialize
	PLG:createFrame()
	if PLG.UseFrame then
		PLG.backframe:Hide()
	end
	PLG.frame:Hide()
	PLG:createLabels()	
	PLG:RegisterEvents()
		
	-- timers
	PLG.frame:SetScript("OnUpdate", function(self, elapsed)	
		PLG.timer1 = PLG.timer1 + elapsed
		PLG.timer5 = PLG.timer5 + elapsed
		-- 1 second timer
		if PLG.timer1 >= 1 then
			PLG.timer1 = 0	
			-- handle TradeSkillDW Queue Frame Popup
			if PLG.TradeSkillDW then 					
				if _G["TradeSkillDW_QueueFrame"] and not PLG.TradeSkillDWQueue then
					PLG.TradeSkillDWQueue = true
					if ( _G["TradeSkillDW_QueueFrame"]:IsShown() ) then
						PLG.backframe:ClearAllPoints()
						PLG.backframe:SetPoint("TOPLEFT", TradeSkillDW_QueueFrame, "TOPRIGHT", 30, 0)
					end
					_G["TradeSkillDW_QueueFrame"]:SetScript("OnShow", function(self, elapsed)
						PLG.backframe:ClearAllPoints()	
						PLG.backframe:SetPoint("TOPLEFT", TradeSkillDW_QueueFrame, "TOPRIGHT", 30, 0)
					end)
					_G["TradeSkillDW_QueueFrame"]:SetScript("OnHide", function(self, elapsed)	
						PLG.backframe:ClearAllPoints()
						PLG.backframe:SetPoint("TOPLEFT", TradeSkillFrame, "TOPRIGHT", 50, 0)
					end)
				end
			end
		end
		-- 5 second timer
		if PLG.timer5 >= 5 then
			PLG.timer5 = 0
			if PLG.waitForServer == true then
				PLG:Update()
				PLG.waitForServer = false
			end
		end
	end)
			
	-- events
	PLG.frame:SetScript("OnEvent", function(self, event, arg1)
		PLG:Debug("OnEvent",event)
		if event == "TRADE_SKILL_SHOW" then
			PLG:TRADE_SKILL_SHOW()
		elseif event == "TRADE_SKILL_CLOSE" then
			PLG:TRADE_SKILL_CLOSE()
		else
			if ( TradeSkillFrame:IsShown() ) and ( PLG.frame:IsShown() ) then -- update if shown and not maxed
				PLG:Update()
			end
			if event == "TRADE_SKILL_UPDATE" and (not PLG.loaded) then -- first run we miss _SHOW
				PLG:TRADE_SKILL_SHOW()
				PLG.loaded = true
			end
		end
	end)
end

---------------------------
-- slash commands
---------------------------
SLASH_PLG1 = "/PLG"
SlashCmdList["PLG"] = function (msg, editBox)
	msg = string.lower(msg)
	if msg == "debug" then
		if PLG.verbose == false then
			PLG.verbose = true
			PLG:sendMSG("Debug Mode: on")
		else
			PLG.verbose = false
			PLG:sendMSG("Debug Mode: off")
		end
	else
		if ( PLG.frame:IsShown() ) then
			PLG:TRADE_SKILL_CLOSE()
			PLG:sendMSG("is hidden")
		else
			PLG:TRADE_SKILL_SHOW(true)
			--if PLG.UseFrame then 
			--	PLG.backframe:Show()
			--else
			--	PLG:MakeDouble()
			--end
			--PLG.frame:Show()
			PLG:sendMSG("is visible")
		end
	end
end

---------------------------
-- launch
---------------------------
PLG:Initialize()